//---------------------------------------------------------------------
// GBALIB: anim.h
// by Omar Cornut
//---------------------------------------------------------------------
// Animation System
//---------------------------------------------------------------------
// A t_animator	is an object playing animations	based on an	animation set.
// Typically, each moving sprite/character within a	game could have	an
// associated t_animator.
// An animation	set	is static (defined in ROM) and contains	a table	of
// animations, which are themselves	a list of frames with associated delay
// and frame offset.
// Note: the animation system is UNSAFE	and	avoid verifying data for
// performance purpose.	Please insure the correctness of your data.
//---------------------------------------------------------------------
// Needs:
//	- boundingbox
//	- sprites
//	- DMA
//---------------------------------------------------------------------

//---------------------------------------------------------------------
// DEFINITIONS
//---------------------------------------------------------------------

// Animator	Mode
#define	ANIMATOR_MODE_OAM_CHANGE_IDX	(0)	// Change character	idx	in OAM
#define	ANIMATOR_MODE_VRAM_COPY_DATA	(1)	// Overwrite characters	to VRAM

// Flipping information, currently hardcored for X flipped only
// The code needs (a few) changes in order to support X and Y flipping in boundingboxes
#define ANIMATOR_BOUNDINGBOX_AXIS_TO_FLIP	(1)	// X
#define ANIMATOR_BOUNDINGBOX_FLIP_MAX		(1 << (ANIMATOR_BOUNDINGBOX_AXIS_TO_FLIP))

//---------------------------------------------------------------------
// STRUCTURES
//---------------------------------------------------------------------

// Static structure: ts_anim_frame
// Define a	single frame delay and data
typedef	struct __attribute__((packed))
{
	u8					delay;				// Delay in	frames (0 =	end	of animation)
	u8					i_picture;			// Picture Index (Tile = Picture * frame_size|idx_increment)
	u8					i_position;			// Position Index
	u8					i_boundingbox;		// Bounding Box Index
}						ts_anim_frame;

// Static packed structure:	ts_anim
// Define an animation
typedef	struct __attribute__((packed))
{
	u8					next_anim;
	u8					next_frame;
	// Repeated	section
	ts_anim_frame		frames;
}						ts_anim;

// Static packed structure: ts_anim_position
// Define an animation frame position
typedef struct __attribute__((packed))
{
	s8					x, y;
}						ts_anim_position;

// Static packed structure:	ts_anims_set
// Define an animation set
typedef	struct __attribute__((packed))
{
	u16					picture_size;			// Picture size (in bytes)
	u16					positions_num;			// Number of Positions data
	u16					positions_offset;		// Offset to Positions data
	u16					boundingboxes_num;		// Number of BoundingBoxes data
	u16					boundingboxes_offset;	// Offset BoundingBoxes data
	// The data below ARE NOT accessible as is in the structure.
	// They are repeated blocks of data.
	u16             	anims_offsets[];		// Repeated	section	(* number of animations)
	// ...
}						ts_anims_set;

// Structure: t_animator_state
// Animator state instance
// Those are the data to be saved/restored in order to perform backup of a running animation
typedef struct
{
	u16					loop_count;				// Loop	counter, increased at the end of an	animation
	u16					delay_current;			// Current delay to	wait until end of frame
	u16					anim_current;			// Current animation within	anims_set
	ts_anim_frame *		frame_current;			// Current frame within	animation
}   					t_animator_state;

// Structure: t_animator
// Animator	instance
typedef	struct
{
	t_animator_state	state;					// Animation state
	u16					mode;					// Animation mode (see ANIMATOR_MODE_* definitions)
	ts_anims_set *		anims_set;				// Pointer to anims_set	in ROM
	t_sprite *			sprite;					// Pointer to high-level sprite (may be NULL if mode == ANIMATOR_MODE_VRAM_COPY_DATA)
	union
	{
		struct
		{
			u16			idx_start;				// Index of	first frame
			u16			idx_increment;			// Size	of each	frame in index increment
		}				mode_oam_change_idx;
		struct
		{
			u16	*		src_rom_tab;			// Pointer to ROM Address where	all	frames are stored
			u16	*		dst_video;				// Pointer to Video	RAM	area where to copy the current frame
		}				mode_vram_copy_data;
	};
}						t_animator;

//---------------------------------------------------------------------
// FUNCTIONS
//---------------------------------------------------------------------

// FUNCTION: animator_init()
// Initialize an animator
void	animator_init(t_animator *a, ts_anims_set *anims_set, t_sprite *sprite,	int mode, void *arg1, void *arg2);

// FUNCTION: animator_start()
// Start given animation at	given frame
void	animator_start(t_animator *a, int anim_index, int frame_index);

// FUNCTION: animator_set()
// Set given animation at given frame
// This is quite redundant with animator_start(), but doesn't update video now
void	animator_set(t_animator *a, int anim_index, int frame_index);

// FUNCTION: animator_set_frame()
// Set current animation to	given frame
void	animator_set_frame(t_animator *a, int frame_index);

// FUNCTION: animator_update()
// Update the animator (play current animation,	etc..)
// Must	be called each time	unit (usually: frame)
void	animator_update(t_animator *a);

// FUNCTION: animator_get_current_boundingbox()
// Return address to the boundingbox data for the current animation frame
t_boundingbox *animator_get_current_boundingbox(t_animator *a);

// FUNCTION: animator_state_backup()
// Backup an animator state
void	animator_state_backup(t_animator *src, t_animator_state *dst);

// FUNCTION: animator_state_restore()
// Restore an animator state
void	animator_state_restore(t_animator *dst, t_animator_state *src);

//---------------------------------------------------------------------
