//---------------------------------------------------------------------
// GBALIB: dma.h
// by Omar Cornut
//---------------------------------------------------------------------
// DMA System
//---------------------------------------------------------------------
// Simple wrapper around hardware DMA.
//---------------------------------------------------------------------

//---------------------------------------------------------------------
// NOTES (from No$GBA technical documentation)
//---------------------------------------------------------------------
// DMA0 		- Highest priority, best for timing critical transfers (eg. HBlank DMA).
// DMA1 & DMA2 	- Can be used to feed digital sample data to the Sound FIFOs.
// DMA3 		- Can be used to write to Game Pak ROM/FlashROM (but not GamePak SRAM).
// Beside for that, each DMA 0-3 may be used for whatever general purposes.
//---------------------------------------------------------------------
// Access Rights:			<Read>			<Write>
// DMA0						Internal		Internal
// DMA1						Anywhere		Internal
// DMA2						Anywhere		Internal
// DMA3						Anywhere		Anywhere
//---------------------------------------------------------------------
// Special timing flag:
// DMA0						Unavailable
// DMA1						Sound FIFO A
// DMA2						Sound FIFO B
// DMA3						Video Capture Mode
//---------------------------------------------------------------------
// After setting the DMA_ENABLE bit, wait 2 clock cycles before accessing
// any DMA related registers.
//---------------------------------------------------------------------

//---------------------------------------------------------------------
// DEFINITIONS
//---------------------------------------------------------------------

#define	DMA_DST_INCREMENT			(0x00000000)	// Destination increment
#define	DMA_DST_DECREMENT			(0x00200000)	// Destination decrement
#define	DMA_DST_FIXED				(0x00400000)	// Destination is fixed
#define	DMA_DST_RELOAD				(0x00600000)	// Destination increment/reload
#define	DMA_SRC_INCREMENT			(0x00000000)	// Source increment
#define	DMA_SRC_DECREMENT			(0x00800000)	// Source decrement
#define	DMA_SRC_FIXED				(0x01000000)	// Source is fixed
#define	DMA_REPEAT					(0x02000000)	// DMA Repeat flag
#define	DMA_16						(0x00000000)	// 16-bits transfers
#define	DMA_32						(0x04000000)	// 32-bits transfers
#define DMA_GAMEPAK_DRQ				(0x08000000)	// DRQ from GamePak (DMA3 only)
#define	DMA_TIMING_IMMEDIATE		(0x00000000)	// Start immediately
#define	DMA_TIMING_VBLANK			(0x10000000)	// Start at	V-Blank
#define	DMA_TIMING_HBLANK			(0x20000000)	// Start at	H-Blank
#define	DMA_TIMING_SPECIAL			(0x30000000)	// See notes above
#define	DMA_IE						(0x40000000)	// Interrupt Enable	at end of DMA
#define DMA_ENABLE					(0x80000000)	// Set to start	DMA

#define	DMA_16_NOW					(DMA_ENABLE	| DMA_TIMING_IMMEDIATE | DMA_16)
#define	DMA_32_NOW					(DMA_ENABLE	| DMA_TIMING_IMMEDIATE | DMA_32)

//---------------------------------------------------------------------
// FUNCTIONS
//---------------------------------------------------------------------

// FUNCTION: CPU_Naive_Memory_Copy()
// Copy memory naively from any location to another.
// NOTE: this function write per 4-bytes blocks, then byte per byte, be careful...
void	CPU_Naive_Memory_Copy(void *dst, void *src, u32 size);

// FUNCTION: DMA_Copy()
// Enable given	channel	DMA, with given	parameters,	source & destination addresses,	size
void	DMA_Copy(int channel, u32 mode,	void *src, void	*dst, u32 size);

//---------------------------------------------------------------------
