 //---------------------------------------------------------------------
// GBALIB: vram.c
// by Omar Cornut
//---------------------------------------------------------------------
// VRAM Manager
//---------------------------------------------------------------------
// This is currently an extremely primitive VRAM Manager
//
// This VRAM Manager does not do any check on allocated data. In most
// cases, the optimal way is then to store mapping at the end of VRAM.
//
// Things to do:
//  - Be able to free blocks of tiles
//  - Be able to specify non-usable memory area (used to store maps, etc..)
//	- Implement a data management system, via usage instance counter
//  - Perform benchs between DMA/16, DMA/32, direct copy, for copying to VRAM
//  - ...
//---------------------------------------------------------------------

#include "gbalib.h"

//---------------------------------------------------------------------
// DATA
//---------------------------------------------------------------------

static  u16 *   vram_bg_pnt;
static  u16 *   vram_obj_pnt;
static  u16     vram_bg_tiles_copied;
static  u16     vram_obj_tiles_copied;

//---------------------------------------------------------------------
// FUNCTIONS
//---------------------------------------------------------------------

// FUNCTION: vram_bg_start()
// Setup subsequent copying at given location
void        vram_bg_start(u16 *addr)
{
    if (addr)
    {
        vram_bg_pnt = addr;
        vram_bg_tiles_copied = (addr - MEM_VRAM_BG) / (TILE_SIZE_16 / sizeof (u16));
    }
    else
    {
        vram_bg_pnt = MEM_VRAM_BG;
        vram_bg_tiles_copied = 0;
    }
}

// FUNCTION: vram_bg_start()
// Allocate memory of given size (in bytes)
// Return starting tile number
u16         vram_bg_alloc(int size)
{
    u16     ret;
    
    ret = vram_bg_tiles_copied;
    vram_bg_tiles_copied += size / TILE_SIZE_16;
    vram_bg_pnt += size / sizeof (u16);
    return (ret);
}

// FUNCTION: vram_bg_copy()
// Allocate memory of given size (in bytes), and copy data from given source
// Return starting tile number
u16         vram_bg_copy(u16 *src, int size)
{
    u16     ret;
    u16 *   addr;
    
    addr = vram_bg_pnt;
    ret = vram_bg_alloc(size);
	DMA_Copy(3, DMA_16_NOW, (void *)src, addr, size / sizeof (u16));
    return (ret);    
}

//---------------------------------------------------------------------

// FUNCTION: vram_obj_start()
// Setup subsequent copying at given location
void        vram_obj_start(u16 *addr)
{
    if (addr)
    {
        vram_obj_pnt = addr;
        vram_obj_tiles_copied = (addr - MEM_VRAM_OBJ) / (TILE_SIZE_16 / sizeof (u16));
    }
    else
    {
        vram_obj_pnt = MEM_VRAM_OBJ;
        vram_obj_tiles_copied = 0;
    }
}

// FUNCTION: vram_obj_start()
// Allocate memory of given size (in bytes)
// Return starting tile number
u16         vram_obj_alloc(int size)
{
    u16     ret;
    
    ret = vram_obj_tiles_copied;
    vram_obj_tiles_copied += size / TILE_SIZE_16;
    vram_obj_pnt += size / sizeof (u16);
    return (ret);
}

// FUNCTION: vram_obj_copy()
// Allocate memory of given size (in bytes), and copy data from given source
// Return starting tile number
u16         vram_obj_copy(u16 *src, int size)
{
    u16     ret;
    u16 *   addr;
    
    addr = vram_obj_pnt;
    ret = vram_obj_alloc(size);
	DMA_Copy(3, DMA_16_NOW, (void *)src, addr, size / sizeof (u16));
    return (ret);    
}

//---------------------------------------------------------------------

// FUNCTION: vram_bg_copy_block()
// Allocate memory of given size (in bytes), and copy data from given source
// Then fill given block with given block informations
void        vram_bg_copy_block(u16 *src, t_bg_block *block, int block_x, int block_y)
{
    u16     ret;
    
    ret = vram_bg_copy(src, block_x * block_y * TILE_SIZE_16);
    block->tiles_idx = ret;
    block->size_x = block_x;
    block->size_y = block_y;
}

// FUNCTION: vram_bg_copy_block_256()
void        vram_bg_copy_block_256(u16 *src, t_bg_block *block, int block_x, int block_y)
{
    u16     ret;
    
    ret = vram_bg_copy(src, block_x * block_y * TILE_SIZE_256);
    block->tiles_idx = ret / 2;
    block->size_x = block_x;
    block->size_y = block_y;
}

//---------------------------------------------------------------------
